@@ -1,10 +1,15 @@
 from __future__ import annotations
-import numpy as np
-from numpy.typing import ArrayLike, DTypeLike, _ShapeLike
 import os
+import sys
 import re
 import glob
 import itertools
+if sys.version_info < (3, 10):
+    from typing_extensions import ParamSpec
+else:
+    from typing import ParamSpec
+import numpy as np
+from numpy.typing import ArrayLike, DTypeLike, _ShapeLike
 from functools import wraps
 
 from .utils.io import open_img, open_as_dask, get_scale_from_meta, open_mrc, open_tif
@@ -164,8 +169,9 @@ def aslazy(arr: ArrayLike,
     
     return self
 
+_P = ParamSpec("_P")
 
-def inject_numpy_function(func: Callable):
+def inject_numpy_function(func: Callable[_P, np.ndarray]) -> Callable[_P, ImgArray]:
     npfunc: Callable = getattr(np, func.__name__)
     @wraps(func)
     def _func(*args, **kwargs):
@@ -233,9 +239,11 @@ def gaussian_kernel(shape: _ShapeLike,
     return ker
 
 
-def circular_mask(radius: nDFloat, 
-                  shape: _ShapeLike,
-                  center: str | tuple[float, ...] = "center") -> ImgArray:
+def circular_mask(
+    radius: nDFloat, 
+    shape: _ShapeLike,
+    center: str | tuple[float, ...] = "center"
+) -> ImgArray:
     """
     Make a circular or ellipsoid shaped mask. Region close to center will be filled with ``False``. 
 
@@ -296,11 +304,13 @@ def sample_image(name: str) -> ImgArray:
 #   Imread functions
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
 
-def imread(path: str,
-           dtype: DTypeLike = None,
-           key: str = None,
-           *, 
-           squeeze: bool = False) -> ImgArray:
+def imread(
+    path: str,
+    dtype: DTypeLike = None,
+    key: str = None,
+    *, 
+    squeeze: bool = False
+) -> ImgArray:
     r"""
     Load image(s) from a path. You can read list of images from directories with wildcards or ``"$"``
     in ``path``.
@@ -335,6 +345,8 @@ def imread(path: str,
         
     """    
     path = str(path)
+    if not os.path.exists(path):
+        raise ValueError(f"Path does not exist: {path}.")
     is_memmap = (key is not None)
     
     if "$" in path:
@@ -437,10 +449,12 @@ def _imread_glob(path: str, squeeze: bool = False, **kwargs) -> ImgArray:
     out.temp = paths
     return out
 
-def _imread_stack(path: str, 
-                  dtype: DTypeLike = None,
-                  key: str = None,
-                  squeeze: bool = False):
+def _imread_stack(
+    path: str, 
+    dtype: DTypeLike = None,
+    key: str = None,
+    squeeze: bool = False
+):
     r"""
     Read separate image files using formated string. This function is useful when files/folders
     are named in a certain rule, such as ".../pos_0/img_0.tif", ".../pos_0/img_1.tif".
@@ -561,8 +575,10 @@ def _imread_stack(path: str,
     return self.sort_axes()
 
 
-def imread_collection(path: str | list[str], 
-                      filt: Callable[[np.ndarray], bool] = None) -> DataList:
+def imread_collection(
+    path: str | list[str], 
+    filt: Callable[[np.ndarray], bool] | None = None,
+) -> DataList:
     """
     Open images as ImgArray and store them in DataList.
 
@@ -599,10 +615,12 @@ def imread_collection(path: str | list[str],
     return arrlist
 
 
-def lazy_imread(path: str, 
-                chunks="default",
-                *, 
-                squeeze: bool = False) -> LazyImgArray:
+def lazy_imread(
+    path: str, 
+    chunks="auto",
+    *, 
+    squeeze: bool = False
+) -> LazyImgArray:
     """
     Read an image lazily. Image file is first opened as an memory map, and subsequently converted
     to `numpy.ndarray` or `cupy.ndarray` chunkwise by `dask.array.map_blocks`.
@@ -622,6 +640,9 @@ def lazy_imread(path: str,
     LazyImgArray
     """    
     path = str(path)
+    if not os.path.exists(path):
+        raise ValueError(f"Path does not exist: {path}.")
+    
     if "*" in path:
         return _lazy_imread_glob(path, chunks=chunks, squeeze=squeeze)
     fname, fext = os.path.splitext(os.path.basename(path))
@@ -654,7 +675,7 @@ def lazy_imread(path: str,
         return self.sort_axes()
 
 
-def _lazy_imread_glob(path:str, squeeze=False, **kwargs) -> LazyImgArray:
+def _lazy_imread_glob(path: str, squeeze: bool = False, **kwargs) -> LazyImgArray:
     """
     Read images recursively from a directory, and stack them into one LazyImgArray.
 
