@@ -1,102 +1,85 @@
 from __future__ import annotations
+from matplotlib import pyplot as plt
 import numpy as np
 from warnings import warn
+from .core import asarray as ip_asarray
 from .arrays import ImgArray, PropArray
 from .arrays._utils import _docs
+from .arrays._utils._transform import polar2d
 from .arrays._utils._corr import subpixel_pcc
 from .utils.axesop import complement_axes, add_axes
 from .utils.utilcls import Progress
 from .utils.deco import dims_to_spatial_axes
-from ._cupy import xp, asnumpy, xp_ndi
+from .array_api import xp
 from ._types import Dims
 
 __all__ = ["fsc", "fourier_shell_correlation", "ncc", "zncc", "fourier_ncc", "fourier_zncc",
-           "nmi", "pcc_maximum", "ft_pcc_maximum", "pearson_coloc", "manders_coloc"]
+           "nmi", "pcc_maximum", "ft_pcc_maximum", "polar_pcc_maximum",
+           "pearson_coloc", "manders_coloc"]
 
 @_docs.write_docs
-@dims_to_spatial_axes
-def fsc(img0: ImgArray, 
-        img1: ImgArray, 
-        nbin: int = 32, 
-        r_max: float = None,
-        *,
-        squeeze: bool = True, 
-        dims: Dims = None) -> PropArray:
+def fsc(
+    img0: ImgArray, 
+    img1: ImgArray, 
+    dfreq: float = 0.02,
+) -> tuple[np.ndarray, np.ndarray]:
     r"""
     Calculate Fourier Shell Correlation (FSC; or Fourier Ring Correlation, FRC, for 2-D images) 
     between two images. FSC is defined as:
     
-        .. math::
-        
-            FSC(r) = \frac{Re(\sum_{r<r'<r+dr}[F_0(r') \cdot \bar{F_1}(r)])}
-            {\sqrt{\sum_{r<r'<r+dr}|F_0(r')|^2 \cdot \sum_{r<r'<r+dr}|F_1(r')|^2}}
+    .. math::
+    
+        FSC(r) = \frac{Re(\sum_{r<r'<r+dr}[F_0(r') \cdot \bar{F_1}(r)])}
+        {\sqrt{\sum_{r<r'<r+dr}|F_0(r')|^2 \cdot \sum_{r<r'<r+dr}|F_1(r')|^2}}
+    
+    In this function, frequency domain will be binned like this:
+    
+    .. code-block::
+
+        |---|---|---|---|---|
+        0  0.1 0.2 0.3 0.4 0.5
+    
+    and frequencies calculated in each bin will be 0.05, 0.15, ..., 0.45.
     
-                  
     Parameters
     ----------
     {inputs_of_correlation}
-    nbin : int, default is 32
-        Number of bins.
-    r_max : float, optional
-        Maximum radius to make profile. Region 0 <= r < r_max will be split into `nbin` rings
-        (or shells). **Scale must be considered** because scales of each axis may vary.
-    {squeeze}
-    {dims}
+    dfreq : float, default is 0.02
+        Difference of frequencies. This value will be the width of bins.
                 
     Returns
     -------
-    PropArray
-        FSC stored in x-axis by default. If input images have tzcyx-axes, then an array with 
-        tcx-axes will be returned. Make sure x-axis no longer means length in x because images
-        are Fourier transformed.
+    Two np.ndarray
+        The first array is frequency, and the second array is FSC.
     """    
     img0, img1 = _check_inputs(img0, img1)
     
-    spatial_shape = img0.sizesof(dims)
-    inds = xp.indices(spatial_shape)
+    shape = img0.shape
+    dims = img0.axes
     
-    center = [s/2 for s in spatial_shape]
-    
-    r = xp.sqrt(sum(((x - c)/img0.scale[a])**2 for x, c, a in zip(inds, center, dims)))
-    r_lim = r.max()
-        
-    # check r_max
-    if r_max is None:
-        r_max = r_lim
-    elif r_max > r_lim or r_max <= 0:
-        raise ValueError(f"`r_max` must be in range of 0 < r_max <= {r_lim} with this image.")
+    freqs = xp.meshgrid(*[xp.fft.fftshift(xp.fft.fftfreq(s)) for s in shape])
+    r = xp.sqrt(sum(f**2 for f in freqs))
     
     with Progress("fsc"):
         # make radially separated labels
-        r_rel = r/r_max
-        labels = (nbin * r_rel).astype(np.uint16)
-        labels[r_rel >= 1] = 0
+        labels = (r/dfreq).astype(np.uint16)
+        nlabels = int(xp.asnumpy(labels.max()))
         
-        c_axes = complement_axes(dims, img0.axes)
-
-        nlabels = int(asnumpy(labels.max()))
-        
-        out = xp.empty(img0.sizesof(c_axes)+(nlabels,), dtype=xp.float32)
+        out = xp.empty(nlabels, dtype=np.float32)
         def radial_sum(arr):
             arr = xp.asarray(arr)
-            return xp_ndi.sum_labels(arr, labels=labels, index=xp.arange(1, nlabels+1))
+            return xp.ndi.sum_labels(arr, labels=labels, index=xp.arange(0, nlabels))
 
         f0 = img0.fft(dims=dims)
         f1 = img1.fft(dims=dims)
         
-        for sl, f0_, f1_ in iter2(f0, f1, c_axes, exclude=dims):
-            cov = f0_.real*f1_.real + f0_.imag*f1_.imag
-            pw0 = f0_.real**2 + f0_.imag**2
-            pw1 = f1_.real**2 + f1_.imag**2
-        
-            out[sl] = radial_sum(cov)/xp.sqrt(radial_sum(pw0)*radial_sum(pw1))
-        
-    if out.ndim == 0 and squeeze:
-        out = out[()]
+        cov = f0.real*f1.real + f0.imag*f1.imag
+        pw0 = f0.real**2 + f0.imag**2
+        pw1 = f1.real**2 + f1.imag**2
     
-    out = PropArray(asnumpy(out), dtype=np.float32, axes=c_axes+dims[-1], 
-                    dirpath=img0.dirpath, metadata=img0.metadata, propname="fsc")
-    return out
+        out = radial_sum(cov)/xp.sqrt(radial_sum(pw0) * radial_sum(pw1))
+    freq = (np.arange(len(out)) + 0.5) * dfreq
+    return freq, xp.asnumpy(out)
 
 # alias
 fourier_shell_correlation = fsc
@@ -111,7 +94,7 @@ def _ncc(img0: ImgArray, img1: ImgArray, dims: Dims):
     img1 = xp.asarray(img1)
     corr = xp.sum(img0 * img1, axis=dims) / (
         xp.std(img0, axis=dims)*xp.std(img1, axis=dims)) / n
-    return asnumpy(corr)
+    return xp.asnumpy(corr)
 
 
 def _masked_ncc(img0: ImgArray, img1: ImgArray, dims: Dims, mask: ImgArray):
@@ -125,16 +108,12 @@ def _masked_ncc(img0: ImgArray, img1: ImgArray, dims: Dims, mask: ImgArray):
         np.ma.std(img0ma, axis=axis)*np.ma.std(img1ma, axis=axis)) / n
 
 
-def _zncc(img0: ImgArray, img1: ImgArray, dims: Dims):
+def _zncc(img0: xp.ndarray, img1: xp.ndarray, dims: Dims):
     # Basic Zero-Normalized Cross Correlation with batch processing.
     # Inputs must be already zero-normalized.
-    if isinstance(dims, str):
-        dims = tuple(img0.axisof(a) for a in dims)
-    img0 = xp.asarray(img0)
-    img1 = xp.asarray(img1)
     corr = xp.sum(img0 * img1, axis=dims) / (
         xp.sqrt(xp.sum(img0**2, axis=dims)*xp.sum(img1**2, axis=dims)))
-    return asnumpy(corr)
+    return corr
 
 
 def _masked_zncc(img0: ImgArray, img1: ImgArray, dims: Dims, mask: ImgArray):
@@ -148,12 +127,14 @@ def _masked_zncc(img0: ImgArray, img1: ImgArray, dims: Dims, mask: ImgArray):
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def ncc(img0: ImgArray, 
-        img1: ImgArray, 
-        mask: ImgArray | None = None, 
-        squeeze: bool = True, 
-        *, 
-        dims: Dims = None) -> PropArray | float:
+def ncc(
+    img0: ImgArray, 
+    img1: ImgArray, 
+    mask: ImgArray | None = None, 
+    squeeze: bool = True, 
+    *, 
+    dims: Dims = None
+) -> PropArray | float:
     """
     Normalized Cross Correlation.
     
@@ -181,12 +162,14 @@ def ncc(img0: ImgArray,
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def zncc(img0: ImgArray, 
-         img1: ImgArray, 
-         mask: ImgArray | None = None,
-         squeeze: bool = True,
-         *,
-         dims: Dims = None) -> PropArray | float:
+def zncc(
+    img0: ImgArray, 
+    img1: ImgArray, 
+    mask: ImgArray | None = None,
+    squeeze: bool = True,
+    *,
+    dims: Dims = None
+) -> PropArray | float:
     """
     Zero-Normalized Cross Correlation.
     
@@ -206,26 +189,31 @@ def zncc(img0: ImgArray,
     """    
     with Progress("zncc"):
         img0, img1 = _check_inputs(img0, img1)
-        img0zn = img0 - np.mean(img0, axis=dims, keepdims=True)
-        img1zn = img1 - np.mean(img1, axis=dims, keepdims=True)
+        dims = tuple(img0.axisof(a) for a in dims)
+        img0 = xp.asarray(img0.value)
+        img1 = xp.asarray(img1.value)
+        img0zn = img0 - xp.mean(img0, axis=dims, keepdims=True)
+        img1zn = img1 - xp.mean(img1, axis=dims, keepdims=True)
         if mask is None:
             corr = _zncc(img0zn, img1zn, dims)
         else:
             corr = _masked_zncc(img0zn, img1zn, dims, mask)
-    return _make_corr_output(corr, img0, "zncc", squeeze, dims)
+    return _make_corr_output(xp.asnumpy(corr), img0, "zncc", squeeze, dims)
 
 # alias
 pearson_coloc = zncc
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def nmi(img0: ImgArray, 
-        img1: ImgArray,
-        mask: ImgArray | None = None,
-        bins: int = 100, 
-        squeeze: bool = True,
-        *, 
-        dims: Dims = None) -> PropArray | float:
+def nmi(
+    img0: ImgArray, 
+    img1: ImgArray,
+    mask: ImgArray | None = None,
+    bins: int = 100, 
+    squeeze: bool = True,
+    *, 
+    dims: Dims = None
+) -> PropArray | float:
     r"""
     Normalized Mutual Information.
     
@@ -269,12 +257,14 @@ def nmi(img0: ImgArray,
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def fourier_ncc(img0: ImgArray, 
-                img1: ImgArray, 
-                mask: ImgArray | None = None, 
-                squeeze: bool = True, 
-                *, 
-                dims: Dims = None) -> PropArray | float:
+def fourier_ncc(
+    img0: ImgArray, 
+    img1: ImgArray, 
+    mask: ImgArray | None = None, 
+    squeeze: bool = True, 
+    *, 
+    dims: Dims = None
+) -> PropArray | float:
     """
     Normalized Cross Correlation in Fourier space.
     
@@ -304,12 +294,14 @@ def fourier_ncc(img0: ImgArray,
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def fourier_zncc(img0: ImgArray, 
-                 img1: ImgArray,
-                 mask: ImgArray | None = None, 
-                 squeeze: bool = True, 
-                 *, 
-                 dims: Dims = None) -> PropArray | float:
+def fourier_zncc(
+    img0: ImgArray, 
+    img1: ImgArray,
+    mask: ImgArray | None = None, 
+    squeeze: bool = True, 
+    *, 
+    dims: Dims = None
+) -> PropArray | float:
     """
     Zero-Normalized Cross Correlation in Fourier space.
     
@@ -329,10 +321,14 @@ def fourier_zncc(img0: ImgArray,
     """    
     with Progress("fourier_zncc"):
         img0, img1 = _check_inputs(img0, img1)
-        f0 = np.sqrt(img0.power_spectra(dims=dims, zero_norm=True))
-        f1 = np.sqrt(img1.power_spectra(dims=dims, zero_norm=True))
-        f0 -= np.mean(f0, axis=dims, keepdims=True)
-        f1 -= np.mean(f1, axis=dims, keepdims=True)
+        dims = tuple(img0.axisof(a) for a in dims)
+        pw0 = xp.asarray(img0.power_spectra(dims=dims, zero_norm=True).value)
+        pw1 = xp.asarray(img1.power_spectra(dims=dims, zero_norm=True).value)
+        
+        f0 = xp.sqrt(pw0)
+        f1 = xp.sqrt(pw1)
+        f0 -= xp.mean(f0, axis=dims, keepdims=True)
+        f1 -= xp.mean(f1, axis=dims, keepdims=True)
         if mask is None:
             corr = _zncc(f0, f1, dims)
         else:
@@ -340,13 +336,18 @@ def fourier_zncc(img0: ImgArray,
     return _make_corr_output(corr, img0, "fourier_zncc", squeeze, dims)
 
 @_docs.write_docs
-def pcc_maximum(img0: ImgArray, 
-                img1: ImgArray, 
-                mask: ImgArray | None = None, 
-                upsample_factor: int = 10,
-                max_shifts: int | tuple[int, ...] | None = None) -> np.ndarray:
+def pcc_maximum(
+    img0: ImgArray, 
+    img1: ImgArray, 
+    mask: ImgArray | None = None, 
+    upsample_factor: int = 10,
+    max_shifts: int | tuple[int, ...] | None = None
+) -> np.ndarray:
     """
-    Calculate lateral shift between two images. Same as ``skimage.registration.phase_cross_correlation``.
+    Calculate lateral shift between two images. 
+    
+    Same as ``skimage.registration.phase_cross_correlation`` but some additional parameters 
+    are supported.
 
     Parameters
     ----------
@@ -354,8 +355,8 @@ def pcc_maximum(img0: ImgArray,
     upsample_factor : int, default is 10
         Up-sampling factor when calculating phase cross correlation.
     max_shifts : int, tuple of int, optional
-        Maximum shifts in each dimension. If a single integer is given, it is interpreted as maximum shifts
-        in all dimensions. No upper bound of shifts if not given.
+        Maximum shifts in each dimension. If a single integer is given, it is interpreted 
+        as maximum shifts in all dimensions. No upper bound of shifts if not given.
     
     Returns
     -------
@@ -370,7 +371,7 @@ def pcc_maximum(img0: ImgArray,
         ft1 = img1.fft(dims=img1.axes)
         if mask is not None:
             ft0[mask] = 0
-        if isinstance(max_shifts, int):
+        if isinstance(max_shifts, (int, float)):
             max_shifts = (max_shifts,) * img0.ndim
         shift = subpixel_pcc(
             xp.asarray(ft0.value), 
@@ -378,16 +379,19 @@ def pcc_maximum(img0: ImgArray,
             upsample_factor, 
             max_shifts=max_shifts
         )
-    return asnumpy(shift)
+    return xp.asnumpy(shift)
 
 @_docs.write_docs
-def ft_pcc_maximum(img0: ImgArray,
-                   img1: ImgArray, 
-                   mask: ImgArray | None = None, 
-                   upsample_factor: int = 10,
-                   max_shifts: int | tuple[int, ...] | None = None) -> np.ndarray:
+def ft_pcc_maximum(
+    img0: ImgArray,
+    img1: ImgArray, 
+    mask: ImgArray | None = None, 
+    upsample_factor: int = 10,
+    max_shifts: float | tuple[float, ...] | None = None
+) -> np.ndarray:
     """
-    Calculate lateral shift between two images. 
+    Calculate lateral shift between two images.
+    
     This function takes Fourier transformed images as input. If you have to repetitively
     use a same template image, this function is faster.
 
@@ -396,8 +400,8 @@ def ft_pcc_maximum(img0: ImgArray,
     {inputs_of_correlation}
     upsample_factor : int, default is 10
         Up-sampling factor when calculating phase cross correlation.
-    max_shifts : int, tuple of int, optional
-        Maximum shifts in each dimension. If a single integer is given, it is interpreted as maximum shifts
+    max_shifts : float, tuple of float, optional
+        Maximum shifts in each dimension. If a single scalar is given, it is interpreted as maximum shifts
         in all dimensions. No upper bound of shifts if not given.
 
     Returns
@@ -410,7 +414,7 @@ def ft_pcc_maximum(img0: ImgArray,
         if mask is not None:
             img0 = img0.copy()
             img0[mask] = 0
-        if isinstance(max_shifts, int):
+        if isinstance(max_shifts, (int, float)):
             max_shifts = (max_shifts,) * img0.ndim
         shift = subpixel_pcc(
             xp.asarray(img0.value), 
@@ -418,15 +422,56 @@ def ft_pcc_maximum(img0: ImgArray,
             upsample_factor,
             max_shifts=max_shifts,
         )
-    return asnumpy(shift)
+    return xp.asnumpy(shift)
+
+@_docs.write_docs
+def polar_pcc_maximum(
+    img0: ImgArray,
+    img1: ImgArray,
+    upsample_factor: int = 10,
+    max_degree: int = None,
+) -> float:
+    """
+    Calculate rotational shift between two images using polar Fourier transformation.
+
+    Parameters
+    ----------
+    {inputs_of_correlation}
+    upsample_factor : int, default is 10
+        Up-sampling factor when calculating phase cross correlation.
+    max_degree : int, tuple of int, optional
+        Maximum rotation in degree.
+
+    Returns
+    -------
+    float
+        Rotation in degree
+    """    
+    img0, img1 = _check_inputs(img0, img1)
+    if img0.ndim != 2:
+        raise TypeError("Currently only 2D image is supported.")
+    if max_degree is None:
+        max_degree = 180
+    rmax = min(img0.shape)
+    with Progress("polar_pcc_maximum_2d"):
+        imgp = ip_asarray(xp.asnumpy(polar2d(img0, rmax, np.pi/180)))
+        imgrotp = ip_asarray(xp.asnumpy(polar2d(img1, rmax, np.pi/180)))
+        max_shifts = (max_degree, 1)
+        shift = pcc_maximum(imgp, imgrotp, upsample_factor=upsample_factor,
+                            max_shifts=max_shifts)
+    # Here, `shift` satisfies `img0.rotate(-shift[0]) == img1`
+    return shift[0]
+
 
 @_docs.write_docs
 @dims_to_spatial_axes
-def manders_coloc(img0: ImgArray, 
-                  img1: np.ndarray,
-                  *,
-                  squeeze: bool = True,
-                  dims: Dims = None) -> PropArray | float:
+def manders_coloc(
+    img0: ImgArray, 
+    img1: np.ndarray,
+    *,
+    squeeze: bool = True,
+    dims: Dims = None
+) -> PropArray | float:
     r"""
     Manders' correlation coefficient. This is defined as following:
     
